@@ -1,15 +1,23 @@
+import future
+from future.builtins import next
+
 import os
-import csv
 import re
 import collections
 import logging
-from cStringIO import StringIO
+from io import StringIO, open
 import sys
 from signal import signal, SIGPIPE, SIG_DFL
-signal(SIGPIPE,SIG_DFL) 
+signal(SIGPIPE, SIG_DFL)
+if sys.version < '3' :
+    import unicodecsv as csv
+else :
+    import csv
+
 
-import AsciiDammit
 import dedupe
+import json
+import argparse
 
 def preProcess(column):
     """
@@ -18,7 +26,6 @@ def preProcess(column):
     Regex. Things like casing, extra spaces, quotes and new lines can
     be ignored.
     """
-    column = AsciiDammit.asciiDammit(column)
     column = re.sub('  +', ' ', column)
     column = re.sub('\n', ' ', column)
     column = column.strip().strip('"').strip("'").lower().strip()
@@ -38,12 +45,13 @@ def readData(input_file, field_names, prefix=None):
     """
 
     data = {}
+    
     reader = csv.DictReader(StringIO(input_file))
     for i, row in enumerate(reader):
         clean_row = [(k, preProcess(v)) for (k, v) in row.items()]
-        if prefix :
-            row_id = "%s|%s" % (prefix, i)
-        else :
+        if prefix:
+            row_id = u"%s|%s" % (prefix, i)
+        else:
             row_id = i
         data[row_id] = dedupe.core.frozendict(clean_row)
 
@@ -69,19 +77,20 @@ def writeResults(clustered_dupes, input_file, output_file):
 
     reader = csv.reader(StringIO(input_file))
 
-    heading_row = reader.next()
-    heading_row.insert(0, 'Cluster ID')
+    heading_row = next(reader)
+    heading_row.insert(0, u'Cluster ID')
     writer.writerow(heading_row)
 
     for row_id, row in enumerate(reader):
-        if row_id in cluster_membership :
+        if row_id in cluster_membership:
             cluster_id = cluster_membership[row_id]
-        else :
+        else:
             cluster_id = unique_record_id
             unique_record_id += 1
         row.insert(0, cluster_id)
         writer.writerow(row)
 
+
 # ## Writing results
 def writeUniqueResults(clustered_dupes, input_file, output_file):
 
@@ -101,25 +110,27 @@ def writeUniqueResults(clustered_dupes, input_file, output_file):
 
     reader = csv.reader(StringIO(input_file))
 
-    heading_row = reader.next()
-    heading_row.insert(0, 'Cluster ID')
+    heading_row = next(reader)
+    heading_row.insert(0, u'Cluster ID')
     writer.writerow(heading_row)
 
     seen_clusters = set()
     for row_id, row in enumerate(reader):
-        if row_id in cluster_membership: 
+        if row_id in cluster_membership:
             cluster_id = cluster_membership[row_id]
-            if cluster_id not in seen_clusters :
+            if cluster_id not in seen_clusters:
                 row.insert(0, cluster_id)
                 writer.writerow(row)
                 seen_clusters.add(cluster_id)
-        else :
+        else:
             cluster_id = unique_record_id
             unique_record_id += 1
             row.insert(0, cluster_id)
             writer.writerow(row)
 
-def writeLinkedResults(clustered_pairs, input_1, input_2, output_file, inner_join = False) :
+
+def writeLinkedResults(clustered_pairs, input_1, input_2, output_file,
+                       inner_join=False):
     logging.info('saving unique results to: %s' % output_file)
 
     matched_records = []
@@ -135,7 +146,7 @@ def writeLinkedResults(clustered_pairs, input_1, input_2, output_file, inner_joi
     length_2 = len(row_header_2)
     row_header += row_header_2
 
-    for pair in clustered_pairs :
+    for pair in clustered_pairs:
         index_1, index_2 = [int(index.split('|', 1)[1]) for index in pair[0]]
 
         matched_records.append(input_1[index_1] + input_2[index_2])
@@ -145,68 +156,75 @@ def writeLinkedResults(clustered_pairs, input_1, input_2, output_file, inner_joi
     writer = csv.writer(output_file)
     writer.writerow(row_header)
 
-    for matches in matched_records :
+    for matches in matched_records:
         writer.writerow(matches)
-   
-    if not inner_join :
 
-        for i, row in enumerate(input_1) :
-            if i not in seen_1 :
-                writer.writerow(row + [None]*length_2)
+    if not inner_join:
 
-        for i, row in enumerate(input_2) :
-            if i not in seen_2 :
-                writer.writerow([None]*length_1 + row)
+        for i, row in enumerate(input_1):
+            if i not in seen_1:
+                writer.writerow(row + [None] * length_2)
+
+        for i, row in enumerate(input_2):
+            if i not in seen_2:
+                writer.writerow([None] * length_1 + row)
+
+class CSVCommand(object) :
+    def __init__(self) :
+        self.parser = argparse.ArgumentParser(
+            formatter_class=argparse.ArgumentDefaultsHelpFormatter)
+
+        self._common_args()
+        self.add_args()
+
+        self.args = self.parser.parse_args()
+
+        self.configuration = {}
+
+        if self.args.config_file:
+            #read from configuration file
+            try:
+                with open(self.args.config_file, 'r') as f:
+                    config = json.load(f)
+                    self.configuration.update(config)
+            except IOError:
+                raise self.parser.error(
+                    "Could not find config file %s. Did you name it correctly?"
+                    % self.args.config_file)
+
+        # override if provided from the command line
+        args_d = vars(self.args)
+        args_d = dict((k, v) for (k, v) in args_d.items() if v is not None)
+        self.configuration.update(args_d)
+
+        self.output_file = self.configuration.get('output_file', None)
+        self.skip_training = self.configuration.get('skip_training', False)
+        self.training_file = self.configuration.get('training_file',
+                                               'training.json')
+        self.sample_size = self.configuration.get('sample_size', 1500)
+        self.recall_weight = self.configuration.get('recall_weight', 1)
+
+        if 'field_definition' in self.configuration:
+            self.field_definition = self.configuration['field_definition']
+        else :
+            self.field_definition = None
 
 
-def consoleLabel(deduper): # pragma : no cover
-    '''
-    Command line interface for presenting and labeling training pairs
-    by the user
-    
-    Argument :
-    A deduper object
-    '''
+    def _common_args(self) :
+        # optional arguments
+        self.parser.add_argument('--config_file', type=str,
+            help='Path to configuration file. Must provide either a config_file or input and field_names.')
+        self.parser.add_argument('--field_names', type=str, nargs="+",
+            help='List of column names for dedupe to pay attention to')
+        self.parser.add_argument('--output_file', type=str,
+            help='CSV file to store deduplication results')
+        self.parser.add_argument('--skip_training', action='store_true',
+            help='Skip labeling examples by user and read training from training_file only')
+        self.parser.add_argument('--training_file', type=str, 
+            help='Path to a new or existing file consisting of labeled training examples')
+        self.parser.add_argument('--sample_size', type=int, 
+            help='Number of random sample pairs to train off of')
+        self.parser.add_argument('--recall_weight', type=int, 
+            help='Threshold that will maximize a weighted average of our precision and recall')
+        self.parser.add_argument('-v', '--verbose', action='count', default=0)
 
-    finished = False
-
-    while not finished :
-        uncertain_pairs = deduper.uncertainPairs()
-
-        labels = {'distinct' : [], 'match' : []}
-
-        for record_pair in uncertain_pairs:
-            label = ''
-            labeled = False
-
-            for pair in record_pair:
-                for field in set(field for field, compare 
-                                 in deduper.data_model.field_comparators) :
-                    line = "%s : %s\n" % (field, pair[field])
-                    sys.stderr.write(line)
-                sys.stderr.write('\n')
-
-            sys.stderr.write('Do these records refer to the same thing?\n')
-
-            valid_response = False
-            while not valid_response:
-                sys.stderr.write('(y)es / (n)o / (u)nsure / (f)inished\n')
-                label = sys.stdin.readline().strip()
-                if label in ['y', 'n', 'u', 'f']:
-                    valid_response = True
-
-            if label == 'y' :
-                labels['match'].append(record_pair)
-                labeled = True
-            elif label == 'n' :
-                labels['distinct'].append(record_pair)
-                labeled = True
-            elif label == 'f':
-                sys.stderr.write('Finished labeling\n')
-                finished = True
-            elif label != 'u':
-                sys.stderr.write('Nonvalid response\n')
-                raise
-
-        if labeled :
-            deduper.markPairs(labels)
